@@ -80,6 +80,13 @@
     #include <fstream>
 #endif
 typedef void (*sig_t_)(int);
+#if !defined(_WIN32) && !defined(_WIN64) && !defined(_MSC_VER)
+    #include <unistd.h>
+#endif
+#ifdef _FOR_R
+    #include <Rcpp.h>
+#endif
+
 
 /* By default, will use Mersenne-Twister for RNG, but can be switched to something faster */
 #ifdef _USE_MERSENNE_TWISTER
@@ -819,13 +826,20 @@ void get_categs(size_t ix_arr[], int x[], size_t st, size_t end, int ncat,
 long double calculate_sum_weights(std::vector<size_t> &ix_arr, size_t st, size_t end, size_t curr_depth,
                                   std::vector<double> &weights_arr, std::unordered_map<size_t, double> &weights_map);
 void set_interrup_global_variable(int s);
+#ifdef _FOR_PYTHON
+bool cy_check_interrupt_switch();
+void cy_tick_off_interrupt_switch();
+#endif
 class SignalSwitcher
 {
 public:
     sig_t_ old_sig;
+    bool is_active;
     SignalSwitcher();
     ~SignalSwitcher();
+    void restore_handle();
 };
+void check_interrupt_switch(SignalSwitcher &ss);
 int return_EXIT_SUCCESS();
 int return_EXIT_FAILURE();
 
