@@ -1546,13 +1546,74 @@ void todense(size_t ix_arr[], size_t st, size_t end,
 /* Function to handle interrupt signals */
 void set_interrup_global_variable(int s)
 {
-    fprintf(stderr, "Error: procedure was interrupted\n");
     #pragma omp critical
     {
         interrupt_switch = true;
     }
 }
 
+void check_interrupt_switch(SignalSwitcher &ss)
+{
+    if (interrupt_switch)
+    {
+        ss.restore_handle();
+        fprintf(stderr, "Error: procedure was interrupted\n");
+        raise(SIGINT);
+        #ifdef _FOR_R
+        Rcpp::checkUserInterrupt();
+        #elif !defined(DONT_THROW_ON_INTERRUPT)
+        throw "Error: procedure was interrupted.\n";
+        #endif
+    }
+}
+
+#ifdef _FOR_PYTHON
+bool cy_check_interrupt_switch()
+{
+    return interrupt_switch;
+}
+
+void cy_tick_off_interrupt_switch()
+{
+    interrupt_switch = false;
+}
+#endif
+
+SignalSwitcher::SignalSwitcher()
+{
+    #pragma omp critical
+    {
+        interrupt_switch = false;
+        this->old_sig = signal(SIGINT, set_interrup_global_variable);
+        this->is_active = true;
+    }
+}
+
+SignalSwitcher::~SignalSwitcher()
+{
+    #pragma omp critical
+    {
+        if (this->is_active)
+            signal(SIGINT, this->old_sig);
+        this->is_active = false;
+        #ifndef _FOR_PYTHON
+        interrupt_switch = false;
+        #endif
+    }
+}
+
+void SignalSwitcher::restore_handle()
+{
+    #pragma omp critical
+    {
+        if (this->is_active)
+        {
+            signal(SIGINT, this->old_sig);
+            this->is_active = false;
+        }
+    }
+}
+
 /* Return the #def'd constants from standard header. This is in order to determine if the return
    value from the 'fit_model' function is a success or failure within Cython, which does not
    allow importing #def'd macro values. */
